package main

import (
	"flag"
	"log"
	"os"
	"os/exec"
	"strconv"
	"sync"
)

var config Config

const sockfile = "/tmp/gopaper.sock"
const pidfilePath = "/tmp/gopaper.pid"

func main() {
	//SETUP FLAGS
	var debugFlag = flag.Bool("debug", false, "help message for flag n")
	flag.Parse()

	//INITIALIZE CONFIG
	err := config.load()
	if err != nil {
		log.Println("unable to initialize config", err)
	}

	//SETUP LOGGER
	log.SetFlags(log.LstdFlags | log.Lshortfile)
	if *debugFlag {
		logfile, err := openLogfile()
		if err != nil {
			log.Println("failed to load config", err)
			cleanExit(1)
		}
		log.SetOutput(logfile)
	}
	log.SetFlags(log.LstdFlags | log.Lshortfile | log.Lmicroseconds)
	log.Println("Welcome to GoPaper!")

	err = killPrevProc()
	if err != nil {
		log.Println("error attempting to kill pre-existing process: ", err)
		cleanExit(1)
	}
	//create new pidfile
	pid := strconv.FormatInt(int64(os.Getpid()), 10)
	log.Printf("pid: %v", pid)
	pidfile, err := os.Create(pidfilePath)
	if err != nil {
		log.Println("error trying to create pidfile:", err)
		cleanExit(1)
	}
	pidfile.WriteString(pid)

	var waitGroup sync.WaitGroup
	waitGroup.Add(1)
	slideshowDir := make(chan string)

	func() {
		defer func() {
			if r := recover(); r != nil {
				log.Println("Panic recovered at top level", r)
				cleanExit(1)
			}
		}()
		go server(slideshowDir)
		go slideshow(slideshowDir)
	}()

	var dir string
	args := os.Args[1:]
	if len(args) > 0 {
		dir = args[0]
	} else {
		dir = config.Default
	}
	slideshowDir <- dir

	waitGroup.Wait()
}

func openLogfile() (*os.File, error) {
	//TODO: log rotation
	f, err := os.OpenFile(config.Cache+"/logfile", os.O_RDWR|os.O_CREATE|os.O_APPEND, 0666)
	if err != nil {
		return nil, err
	}
	return f, nil
}

func cleanExit(code int) {
	if code == 1 {
		log.Println("shutting down gracefully after errors")
	} else {
		log.Println("shutting down gracefully")
	}

	err := os.Remove(sockfile)
	if err != nil && !os.IsNotExist(err) {
		log.Printf("unable to remove sockfile at %v. Error: %v\n", sockfile, err)
	} else if !os.IsNotExist(err) {
		log.Println("sockfile removed")
	} else {
		log.Println("no need to remove sockfile -- it doesn't exist")
	}

	err = os.Remove(pidfilePath)
	if err != nil && !os.IsNotExist(err) {
		log.Printf("unable to remove pidfile at %v. Error: %v\n", pidfilePath, err)
	} else if !os.IsNotExist(err) {
		log.Println("pidfile removed")
	} else {
		log.Println("no need to remove pidfile -- it doesn't exist")
	}
	os.Exit(code)
}

func killPrevProc() error {
	//check if pidfile already exists
	if _, err := os.Stat(pidfilePath); os.IsNotExist(err) {
		return nil
	}
	//if it does, kill the process and delete the pidfile
	prexPid, err := os.ReadFile(pidfilePath)
	if err != nil {
		log.Println("unable to read pre-existing pidfile", err)
		return err
	}
	//kill process
	err = exec.Command("kill", string(prexPid)).Run()
	if err != nil {
		log.Printf("Error attempting to kill process %v: %v\n", string(prexPid), err)
	}
	//remove pidfile
	err = os.Remove(pidfilePath)
	if err != nil {
		log.Printf("unable to remove pidfile %v\n", pidfilePath)
	}
	return nil
}
